%----
% Copyright (C) 1996, Gregor Meyer, FernUniversitaet Hagen, Germany.
% This file is part of the system 'Typical'.
% See the file LICENSE for conditions of distribution and use.
%----
%:- multifile (subtype)/1, (suptype)/1, (type)/1, (pred)/1, (type_import)/1, (local)/1.
















:- ensure_loaded(typeops).

:- module_interface(csp_lib).

:- import(prolog_types).
:- import(csppo).

:- pred test_constraints(poset(@T),list(po_constraint(T))).
:- pred check_cycle(list(order1(T)),list(order1(T))).

:- end_interface(csp_lib).

:- module_body(csp_lib).

:- import(lib).

%----
% test_constraints( +PO, +C) :-
%       checks if constraints C are satisfied in poset PO,
%       PO must be a quasi-lattice.
%       C may not contain logical variables when called.
%
test_constraints(_,[]).
test_constraints(PO,[X =< Y | Tail]) :-
	po_test_leq(PO,X,Y),
	test_constraints(PO,Tail).
test_constraints(PO,[lub(X,Y) | Tail]) :-
	test_lub_list(PO,Y,X),
	test_constraints(PO,Tail).

% test_lub_list(+PO, +List, -LUB) :-
%       computes the least upper bound LUB of the values in LIST
%       over the domain and ordering PO
%
:-  pred test_lub_list(poset(@T),list(T),T).
%
test_lub_list(PO,[X,Y],Z) :-
	po_lub(PO,X,Y,Z).
test_lub_list(PO,[X|Y],Z) :-
	test_lub_list(PO,Y,V),
	po_lub(PO,X,V,Z).


%----
% check_cycle(Order,Cycle) :-					
%       true, if the order relations Order contain 
%       a cycle Cycle, otherwise Cycle is []
%
check_cycle(Ord,Cycle) :-
	check_cycle1(Ord,Ord,Cycle),
	Cycle \== [].

% check_cycle1(Order,OrderOrig,Cycle) :-
%       for each element of Order, test if that element is
%       part of a cycle Cycle in OrderOrig
  
:- pred check_cycle1(list(order1(T)),list(order1(T)),list(order1(T))).

check_cycle1([],_,[]).
check_cycle1([O|_],Ord,Cycle) :-
	check_cycle2(O,Ord,Cycle).
check_cycle1([_|R],Ord,Cycle) :-
        check_cycle1(R,Ord,Cycle).

% check_cycle2(Orderrel,Order,Cycle) :-
%       true, if Orderrel is part of a cycle Cycle in Order
 
:- pred check_cycle2(order1(T),list(order1(T)),list(order1(T))).

check_cycle2(_,[],[]).
check_cycle2(X =< Y,Ord,[X =< Y|Cycle]) :-
	delete(Ord, X =< Y, Nord),
	path(Y, [X], _, Nord,Cycle).

% path(Start, Path, Visited, Order, Ordercycle) :-
%       true, if there is a path from Start to the first element of Path in Order
%       no element of Visited is visited twice
%       Ordercycle is the path expressed in order relations

:- pred path(T,list(T),list(T),list(order1(T)),list(order1(T))).

path(U, [V|Visited], [U, V|Visited], Ord,[U =< V]) :-
	member(U =< V, Ord),
	non_member(U, [V|Visited]).
path(U, [V|Visited], Path, Ord,[W =< V|OrdC]) :-
	member(W =< V, Ord),
	non_member(W, [V|Visited]),
	path(U, [W,V|Visited], Path, Ord,OrdC).

:- end_module(csp_lib).